%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%    k-t acceleration demo ( Cartesian undersampling )
%
%               written by Taehoon Shin  Nov 14, 2005
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;

disp('k-t acceleration demo (Cartesian undersampling)');
disp('');
disp('');

load rawphantom;
n = 64; % image size
n_frame = 40; % number of time frames
del_t = 0.05;   % time interval between two adjacent images  

disp( 'Press any key to begin' );pause; 
clc

disp('[ Reference raw data from 2DFT dynamic imaging ]');
disp('A total of 40 dynamic images of 64 x 64 matrix are sampled in k-t space.');
disp('Note that all 64 phase encoding lines are assumed to be taken simultaneously, i.e. motion artifacts are ignored.');
h_fig1 = figure(1);
set( h_fig1, 'numbertitle', 'off', 'name', 'Reference raw data in k-t space' );
for nn = 1:40
    tt = [ 't= ', num2str( del_t * (nn-1) ) ];
    subplot(5, 8, nn); dispimage( abs( d_raw_truth(:,:,nn) ) );title( tt );
end

disp( 'Press any key to continue' );pause; 
clc

disp('[ Reference images from 2DFT dynamic imaging ]');
disp('Note that only central ring signal keeps changing while the remaining parts are stationary.');
h_fig2 = figure(2);
set( h_fig2, 'numbertitle', 'off', 'name', 'Reference images in xyt space' );
for nn = 1:n_frame
    tt = [ 't= ', num2str( del_t * (nn-1) ) ];
    im_truth(:,:, nn) = ift( d_raw_truth(:,:, nn) );
    subplot(5, 8, nn); dispimage( abs( im_truth(:,:,nn) ) );title( tt );
end

disp( 'Press any key to continue' );pause; 
clc

disp('[ Sampling function for 2-fold undersampling]');
disp('Only odd or even order encoding lines have unit values altenately.');
disp('For clear desciption only the first 16 time frames are displayed.');
h_fig3 = figure(3);
set( h_fig3, 'numbertitle', 'off', 'name', 'Sampling function for 2-fold undersampling in k-t space' );
for nn = 1:n_frame
    tt = [ 't= ', num2str( del_t * (nn-1) ) ];
    if mod( nn, 2) == 1
        samp_func_kt(:,:,nn) = s_odd;
    else
        samp_func_kt(:,:,nn) = s_even;
    end
    samp_func_xt(:,:, nn) = ift( reshape( samp_func_kt(:,:,nn), n,n ) );
    if nn<17
        subplot(4, 4, nn); dispimage( abs( samp_func_kt(:,:,nn) ) );title( tt );
    end
end
disp( 'Press any key to continue' );pause; 
clc

disp('[ Undersampled raw data ]');
disp('Reference raw data are undersampled according to the sampling function displayed just before.');
disp('Encoding lines which are not sampled should be zero-padded.');
disp('Also only the first 16 time frames are displayed for clear desciption.');

h_fig4 = figure(4);
set( h_fig4, 'numbertitle', 'off', 'name', 'Undersampled raw data in k-t space' );
for nn = 1:n_frame
    tt = [ 't= ', num2str( del_t * (nn-1) ) ];
    if mod( nn, 2) == 1
        d_raw_und(:,:,nn) = s_odd .* d_raw_truth(:,:, nn );
    else
        d_raw_und(:,:,nn) = s_even .* d_raw_truth(:,:, nn );;
    end
    if nn < 17
        subplot(4, 4, nn); dispimage( abs( d_raw_und(:,:,nn) ) );title( tt );
    end
    
end
disp( 'Press any key to continue' );pause; 
clc

disp('[ Reconstructed images from 2-fold undersampling ]');
disp('Since the raw data is undersampled in phase encoding direction, aliasing occurs in the same direction.');
h_fig5 = figure(5);
set( h_fig5 , 'numbertitle', 'off', 'name', 'Aliased images from 2-fold undersampling in xyt space.' );
for nn = 1:n_frame
    tt = [ 't= ', num2str( del_t * (nn-1) ) ];
    im_und(:,:,nn) = ift( reshape( d_raw_und(:,:,nn), n,n ) );
    subplot(5, 8, nn); dispimage( abs( im_und(:,:,nn) ) );title( tt );
end

disp( 'Press any key to continue' );pause;
clc

disp('Converting data in xyt space to xyf space using 1D FFT along temporal dimension ...')
for ii = 1 : n
    for jj= 1: n
        xyf_und( ii, jj, : ) = fftshift( fft( im_und( ii, jj, : ) ) ) ;    % Reference object in xyf space
        xyf_truth( ii, jj, : ) = fftshift( fft ( im_truth( ii, jj, : ) ) ) ;   % Undersampled object in xyf space
        psf( ii, jj, : ) = fftshift(  fft( samp_func_xt(ii, jj, : ) ) ); % Point spead function in xyf space
    end
end

disp( 'Press any key to continue' );pause; 
clc

disp('[ 3D plot of reference object in xyf space ]');
disp('The 2D plane accupying full FOV represents DC signal.');
disp('Note that only central part of circular shape has high frequency components.'); 
scatter3D_plot( xyf_truth, n, n_frame, 'Reference object in xyf space', 1.0 );

disp( 'Press any key to continue' );pause; 
clc

disp('[ 3D plot of point spread function in xyf space ]');
disp('The impulse at center is mainlobe, and the other at Nyquist frequency is sidelobe from which aliases are produced.');
disp('Note that convolution with this PSF should be circular. That is, the sidelobe should be considered to be periodic along all three dimensions.');
scatter3D_plot( psf, n, n_frame, 'Point spread function in xyf space', 0.1 );

disp( 'Press any key to continue' );pause; 
clc

disp('[ 3D plot of aliased object in xyf space ]');
disp('Resulting aliased object comes from the 3D convolution between the reference object and PSF.');
scatter3D_plot( xyf_und, n, n_frame, 'Aliased object in xyf space', 0.5 );

disp( 'Press any key to continue' );pause; 
clc


disp('Filtering aliased object in xyf space...');
mask = zeros(n,n); mask( n/4 +1 : 3*n/4, : ) = ones( n/2, n );
for nn = 1 : n_frame
    rec_xyf(:,:, nn) = mask .* xyf_und(:,:, nn);
end
rec_xyf(:,:, 1) = zeros( n,n ); rec_xyf(:,:, n_frame /2 + 1 ) = xyf_und(:,:, n_frame/2 + 1 ); 

disp('Converting filtered data in xyf space to xyt space by FFT along temporal dimension ...');
for ii = 1 : n
    for jj= 1: n
        im_rec( ii, jj, : ) = ifft( fftshift( rec_xyf( ii, jj, : ) ) ) ;
    end
end

disp( 'Press any key to continue' );pause; 
clc

disp('[ Reconstructed images ]');
h_fig6 = figure(6);
set( h_fig6, 'numbertitle', 'off', 'name', 'Reconstructed images in xyt space' );
for nn = 1:n_frame
    tt = [ 't= ', num2str( del_t * (nn-1) ) ];
    subplot(5, 8, nn); dispimage( abs( im_rec(:,:,nn) ) );title( tt );
end





